#!/usr/bin/python

import yasara,os,re,subprocess
from python2to3 import *

# amino acid residue list
aas = ['A','C','D','E','F','G','H','I','K','L','M','N','P','Q','R','S','T','V','W','Y']

def WriteOptionsFile(cachedir,runs,temperature,ph,ionicstrength,vdwdesign):
  optionsfile = open(os.path.join(cachedir,"options_foldx.txt"),"w")
  optionsfile.write("<TITLE>FOLDX_optionfile;\n")
  optionsfile.write("<Temperature>"+str(temperature)+";\n")
  optionsfile.write("<IonStrength>"+str(ionicstrength)+";\n")
  optionsfile.write("<ph>"+str(ph)+";\n")
  optionsfile.write("<VdWDesign>"+str(vdwdesign)+";\n")
  optionsfile.write("<numberOfRuns>"+str(runs)+";\n")
  optionsfile.write("<OutPDB>true;\n")
  optionsfile.close()

def writeCommandsFile_Repair(cachedir):
  commandfile = open(os.path.join(cachedir,"commands_repair.txt"),"w")
  commandfile.write("<TITLE>FOLDX_commandfile;\n<RepairPDB>#;\n<END>;")
  commandfile.close()

def writecommandsfile_BuildModel(cachedir,mutation):
  commandfile = open(os.path.join(cachedir,"commands_BuildModel.txt"),"w")
  commandfile.write("<TITLE>FOLDX_commandfile;\n<BuildModel>"+mutation+",individual_list.txt;\n<END>;")
  commandfile.close()

def repair(cachedir,foldxbin,objectnumber):
  # save the object as PDB for FoldX input
  pdbfilename = "Object"+str(objectnumber)+".pdb"
  yasara.SavePDB(objectnumber,os.path.join(cachedir,pdbfilename))
  # remove the REMARK lines as FoldX crashed on those
  lines = open(os.path.join(cachedir,pdbfilename),"r").readlines()
  output = open(os.path.join(cachedir,pdbfilename),"w")
  for line in lines:
    if line[0:6] != "REMARK":
      output.write(line)
  output.close()
  flag_repair=0
  # make the COMMAND file
  writeCommandsFile_Repair(cachedir)
  # build the RepairPDB command
  repaircommand = foldxbin + " -manual " + pdbfilename + " options_foldx.txt commands_repair.txt"
  # change to temp folder
  os.chdir(cachedir)
  # show RepairPDB message
  yasara.ShowMessage("Repairing Object "+str(objectnumber)+". This can take a few minutes.")
  # run RepairPDB
  os.system(repaircommand)
  try:
    open(os.path.join(cachedir,"RepairPDB_"+pdbfilename),"r").readlines()
    flag_repair=0
  except:
    yasara.plugin.end("Cannot read FoldX RepairPDB output.")
  yasara.HideMessage()
  return(flag_repair)

def writeindividuallist(cachedir,ori,pos,mut,mol):
  individuallist=open(os.path.join(cachedir,"individual_list.txt"),"w") 
  individuallist.write(ori+str(mol)+str(pos)+str(mut)+";")
  individuallist.close()

def runbuildmodel(cachedir,foldxbin,objectnumber,mutation):
  
  output=""
  pdbfilename="RepairPDB_Object"+objectnumber+".pdb"
  ##make the command file
  writecommandsfile_BuildModel(cachedir,mutation)
  ## make the buildmodel command
  buildmodelcommand=foldxbin + " -manual " + pdbfilename + " options_foldx.txt commands_BuildModel.txt"
  ## change to temp folder
  os.chdir(cachedir)
  # show BuildModelPDB message
  yasara.ShowMessage("Found mutations that lower the aggregation tendency. Now running FoldX BuildModel. This can take a few minutes. ")
  # run BuildModel
  os.system(buildmodelcommand)
  ##remove file
  os.remove(os.path.join(cachedir,mutation))
  ## calculate Effect on energy: destabilizing of stabilizing (cuf-off = -0.5)
  ddg=calculateddG(cachedir,mutation)
  return(ddg)

def calculateddG(cachedir,mutation):
  ddg_output="Average_"+mutation
  lines = open(os.path.join(cachedir,ddg_output),"r").readlines()
  ddG=float(lines[9].split("\t")[2])
  return ddG